from abc import abstractmethod
from datetime import datetime

from socketd.transport.core import Entity
from socketd.transport.core.Message import Message
from socketd.transport.core.Session import Session

from folkmq.client.MqMessage import MqMessageBase
from folkmq.common.MqConstants import MqConstants
from folkmq.common.MqTopicHelper import MqTopicHelper
from folkmq.common.MqUtils import MqUtils


class MqMessageReceived(MqMessageBase):
    # 主题
    @abstractmethod
    def getTopic(self) -> str:
        ...

    # 内容
    @abstractmethod
    def getBodyAsString(self) -> str:
        ...

    # 消费者组
    @abstractmethod
    def getConsumerGroup(self) -> str:
        ...

    # 已派发次数
    @abstractmethod
    def getTimes(self) -> int:
        ...

    # 回执
    @abstractmethod
    def acknowledge(self, isOk: bool):
        ...

    # 响应
    @abstractmethod
    def response(self, entity:Entity):
        ...

class MqMessageReceivedImpl(MqMessageReceived):
    def __init__(self, clientInternal:'MqClientInternal', session:Session, source:Message):
        self._clientInternal = clientInternal
        self._session = session
        self._source = source

        mr  = MqUtils.getOf(source)

        self._sender = mr.getSender(source)

        self._key = mr.getKey(source)
        self._tag = mr.getTag(source)
        self._fullTopic = mr.getTopic(source)
        self._topic = MqTopicHelper.getTopic(self._fullTopic)
        self._consumerGroup = mr.getConsumerGroup(source)

        self._qos = mr.getQos(source)
        self._times = mr.getTimes(source)
        self._sequence = mr.isSequence(source)
        self._transaction = mr.isTransaction(source)

        self._expirationL = mr.getExpiration(source)
        self._expiration = self._expirationL

    def getSource(self) -> Message:
        return self._source

    def getSender(self) -> str | None:
        return self._sender

    def getKey(self) -> str:
        return self._key

    def getTag(self) -> str:
        return self._tag

    def getTopic(self) -> str:
        return self._topic

    def getFullTopic(self)->str:
        return self._fullTopic

    def getConsumerGroup(self) -> str:
        return self._consumerGroup

    def getBody(self) -> bytes:
        return self._source.data_as_bytes()
    def getBodyAsString(self) -> str:
        return self._source.data_as_string()

    def getQos(self) -> int:
        return self._qos

    def getAttr(self, name: str) -> str | None:
        return self._source.meta(MqConstants.MQ_ATTR_PREFIX + name)

    def getExpiration(self) -> datetime | None:
        return self._expiration

    def isTransaction(self) -> bool:
        return self._transaction

    def isSequence(self) -> bool:
        return self._sequence

    def getTimes(self) -> int:
        return self._times

    def acknowledge(self, isOk: bool):
        self._clientInternal.reply(self._session, self._source, self, isOk, None)

    def response(self, entity:Entity):
        self._clientInternal.reply(self._session, self._source, self, True, entity)

    def __str__(self) ->str:
        return "MqMessageReceived{" + \
            "key='" + self._key + '\'' + \
            ", tag='" + self._tag + '\'' + \
            ", topic='" + self._topic + '\'' + \
            ", body='" + self.getBodyAsString() + '\'' + \
            '}'





from folkmq.client.MqClient import MqClient
from folkmq.client.MqClientDefault import MqClientDefault
from folkmq.client.MqMessage import MqMessage


class FolkMQ:
    @staticmethod
    def versionCode()->int:
        return 2

    @staticmethod
    def versionCodeAsString()->str:
        return f'{FolkMQ.versionCode()}'

    @staticmethod
    def versionName()->str:
        return "1.4.3"

    @staticmethod
    def createClient(*serverUrls) -> MqClient:
        return MqClientDefault(serverUrls)







from socketd.transport.core import Entity
from socketd.transport.core.EntityMetas import EntityMetas
from socketd.transport.core.Flags import Flags
from socketd.transport.core.Message import Message
from socketd.transport.core.entity.EntityDefault import EntityDefault
from socketd.transport.core.entity.MessageBuilder import MessageBuilder
from socketd.utils.StrUtils import StrUtils

from folkmq.client.MqMessage import MqMessage
from folkmq.common.MqConstants import MqConstants
from folkmq.common.MqMetasResolver import MqMetasResolver
from folkmq.common.MqMetasV2 import MqMetasV2


class MqMetasResolverV2(MqMetasResolver):
    def version(self) -> int:
        return 2

    def getSender(self, m: Entity) -> str:
        return m.meta_or_default(MqMetasV2.MQ_META_SENDER, "")

    def getKey(self, m: Entity) -> str:
        return m.meta_or_default(MqMetasV2.MQ_META_KEY, "")

    def getTag(self, m: Entity) -> str:
        return m.meta_or_default(MqMetasV2.MQ_META_TAG, "")

    def getTopic(self, m: Entity) -> str:
        return m.meta_or_default(MqMetasV2.MQ_META_TOPIC, "")

    def getConsumerGroup(self, m: Entity) -> str:
        return m.meta_or_default(MqMetasV2.MQ_META_CONSUMER_GROUP, "")

    def setConsumerGroup(self, m: Entity, consumerGroup: str):
        m.put_meta(MqMetasV2.MQ_META_CONSUMER_GROUP, consumerGroup)

    def getQos(self, m: Entity) -> int:
        return  0 if "0" == (m.meta(MqMetasV2.MQ_META_QOS)) else 1

    def getTimes(self, m: Entity) -> int:
        return int(m.meta_or_default(MqMetasV2.MQ_META_TIMES, "0"))

    def setTimes(self, m: Entity, times: int):
        m.put_meta(MqMetasV2.MQ_META_TIMES, str(times))

    def getExpiration(self, m: Entity) -> float:
        return float(m.meta_or_default(MqMetasV2.MQ_META_EXPIRATION, "0"))

    def setExpiration(self, m: Entity, expiration: int):
        if  expiration is None:
            m.del_meta(MqMetasV2.MQ_META_EXPIRATION)
        else:
            m.put_meta(MqMetasV2.MQ_META_EXPIRATION, str(expiration))

    def getScheduled(self, m: Entity) -> int:
        return int(m.meta_or_default(MqMetasV2.MQ_META_SCHEDULED, "0"))

    def setScheduled(self, m: Entity, scheduled: int):
        m.put_meta(MqMetasV2.MQ_META_SCHEDULED, str(scheduled))

    def isSequence(self, m: Entity) -> bool:
        return "1" == (m.meta(MqMetasV2.MQ_META_SEQUENCE))

    def isTransaction(self, m: Entity) -> bool:
        return "1" == (m.meta(MqMetasV2.MQ_META_TRANSACTION))

    def setTransaction(self, m: Entity, isTransaction: bool):
        m.put_meta(MqMetasV2.MQ_META_TRANSACTION, ("1" if isTransaction else "0"))

    def publishEntityBuild(self, topic: str, message: MqMessage) -> EntityDefault:
        #构建消息实体
        entity = EntityDefault().data_set(message.getBody())

        entity.meta_put(MqMetasV2.MQ_META_KEY, message.getKey())
        entity.meta_put(MqMetasV2.MQ_META_TOPIC, topic)
        entity.meta_put(MqMetasV2.MQ_META_QOS, ("0" if message.getQos() == 0 else "1"))

        # 标签
        if message.getTag():
            entity.meta_put(MqMetasV2.MQ_META_TAG, message.getTag())

        #定时派发
        if message.getScheduled() is None:
            entity.meta_put(MqMetasV2.MQ_META_SCHEDULED, "0")
        else:
            entity.meta_put(MqMetasV2.MQ_META_SCHEDULED, str(message.getScheduled()))

        # 过期时间
        if message.getExpiration() is not None:
            entity.meta_put(MqMetasV2.MQ_META_EXPIRATION, str(message.getExpiration()))

        if message.isTransaction():
            entity.meta_put(MqMetasV2.MQ_META_TRANSACTION, "1")

        if message.getSender():
            entity.meta_put(MqMetasV2.MQ_META_SENDER, message.getSender())

        # 是否有序
        if message.isSequence() or message.isTransaction():
            entity.at(MqConstants.BROKER_AT_SERVER_HASH)
            if message.isSequence():
                entity.meta_put(MqMetasV2.MQ_META_SEQUENCE, "1");

                if message.isTransaction() == False and message.getSequenceSharding():
                    # 不是事务，并且有顺序分片
                    entity.meta_put(EntityMetas.META_X_Hash, message.getSequenceSharding())
        else:
            entity.at(MqConstants.BROKER_AT_SERVER)

        from folkmq.FolkMQ import FolkMQ
        entity.meta_put(MqMetasV2.MQ_META_VID, str(FolkMQ.versionCode()))

        #用户属性
        for kv in message.getAttrMap():
            entity.put_meta(MqConstants.MQ_ATTR_PREFIX + kv[0], kv[1])

        return entity

    def routingMessageBuild(self, topic: str, message: MqMessage) -> Message:
        entity = self.publishEntityBuild(topic, message)
        messageDefault = (MessageBuilder()
                          .flag(Flags.Message)
                          .sid(StrUtils.guid())
                          .event(MqConstants.MQ_EVENT_PUBLISH)
                          .entity(entity)
                          .build())
        return messageDefault


import abc

from socketd.transport.core import Entity
from socketd.transport.core.Message import Message
from socketd.transport.core.entity.EntityDefault import EntityDefault

from folkmq.client.MqMessage import MqMessage


class MqMetasResolver:
    #
    # 版本号
    #
    @abc.abstractmethod
    def version(self)-> int:...

    #
    # 获取发送人
    #
    @abc.abstractmethod
    def getSender(self, m: Entity)-> str:...

    #
    # 获取主建
    #
    @abc.abstractmethod
    def getKey(self, m: Entity)-> str:...

    #
    # 获取标签
    #
    @abc.abstractmethod
    def getTag(self, m: Entity)-> str:...

    #
    # 获取主题
    #
    @abc.abstractmethod
    def getTopic(self, m: Entity)-> str:...

    #
    # 获取消费者组
    #
    @abc.abstractmethod
    def getConsumerGroup(self, m: Entity)-> str:...

    #
    # 设置消费者组
    #
    @abc.abstractmethod
    def setConsumerGroup(self, m: Entity, consumerGroup: str):...

    #
    # 获取质量等级（0或1）
    #
    @abc.abstractmethod
    def getQos(self, m: Entity)-> int:...

    #
    # 获取派发次数
    #
    @abc.abstractmethod
    def getTimes(self, m: Entity)-> int:...

    #
    # 设置派发次数
    #
    @abc.abstractmethod
    def setTimes(self, m: Entity, times: int):...

    #
    # 获取过期时间
    #
    @abc.abstractmethod
    def getExpiration(self, m: Entity)->float:...

    #
    # 设置过期时间
    #
    @abc.abstractmethod
    def setExpiration(self, m: Entity, expiration: int):...

    #
    # 获取定时时间
    #
    @abc.abstractmethod
    def getScheduled(self, m: Entity)-> int:...

    #
    # 设置定时时间
    #
    @abc.abstractmethod
    def setScheduled(self, m: Entity, scheduled: int):...

    #
    # 是否有序
    #
    @abc.abstractmethod
    def isSequence(self, m: Entity)->bool:...

    #
    # 是否事务
    #
    @abc.abstractmethod
    def isTransaction(self, m: Entity)->bool:...

    #
    # 设置事务
    #
    @abc.abstractmethod
    def setTransaction(self, m: Entity, isTransaction: bool):...

    #
    # 发布实体构建
    #
    # @param topic   主题
    # @param message 消息
    #
    @abc.abstractmethod
    def publishEntityBuild(self, topic: str, message: MqMessage)-> EntityDefault:...

    #
    # 路由消息构建
    #
    # @param topic   主题
    # @param message 消息
    #
    @abc.abstractmethod
    def routingMessageBuild(self, topic: str, message: MqMessage)-> Message:...